package coilPred;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.EnumMap;
import java.util.HashMap;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;

import sequences.CoilState;
import sequences.CoilType;
import sequences.Family;
import sequences.Heptad;
import sequences.Sequence;

public class CoilPredictor {
	static String path = CoilPredictor.class.getProtectionDomain().getCodeSource().getLocation().toString();
	//static String base_dir = path.substring(path.indexOf(":")+1, path.lastIndexOf("/")+1);
	static String base_dir = "/home/jason/Files/coiledcoil";
	static String train_dir = base_dir + "/training_data/";
	static String results_dir = base_dir + "/results/";

	public static void main(String[] args) throws IOException, InterruptedException, ParseException {
		Options options = new Options();
		System.out.println("classpath=" + System.getProperty("java.class.path"));
		options.addOption("sfile",true,"file containing the test sequences");
		options.addOption("out",true,"location to print the result");
		options.addOption("train",false,"train the MRF");
		options.addOption("test",false,"test the MRF");
		CommandLineParser parser = new PosixParser();
		try {
			CommandLine cmd = parser.parse(options,args);
			if (cmd.hasOption("sfile")) {
				String seq_file = cmd.getOptionValue("sfile");
				ArrayList<Sequence> seqs = Parser.fileToSeqs(new File(seq_file));
				CoilHMM myHMM = new CoilHMM();

				myHMM.load_hmm(base_dir + "/multicoilhmm.m");
				String out;
				if(cmd.hasOption("out")) {
					out = cmd.getOptionValue("out");
				} else {
					out = results_dir + "/data.out";
				}
				int seq_num = 1;
				
				//If path already exists, warn and delete file:
				File f = new File(out);
				if (f.exists()) {
					System.out.println("Warning: file already exists. Overwriting.\n" + out);
					f.delete();
				}
				
				for (Sequence seq:seqs) {
					if (seq_num % 10 == 0) {
						System.out.println("Processing sequence " + seq_num);
					}
					printCoil(myHMM, out, seq);
					seq_num++;
				}
			} else if (cmd.hasOption("train")){
				ArrayList<Family> dimerFams = Parser.dirToFams(train_dir+"/dimer/", CoilType.DIMER);
				ArrayList<Family> trimerFams = Parser.dirToFams(train_dir+"/trimer/", CoilType.TRIMER);
				ArrayList<Sequence> negatives = Parser.dirToSeqs(train_dir+"/negative_train/");
				CoilHMM myHMM = new CoilHMM();
				myHMM.generate_hmm2(base_dir,"/leaveOneOutReg_complete.txt",dimerFams, trimerFams, negatives);
				myHMM.write_hmm(base_dir + "/multicoilhmm.m");
			} else if (cmd.hasOption("test")) {
				ArrayList<Family> dimerFams = Parser.dirToFams(train_dir + "/dimer/", CoilType.DIMER);
				ArrayList<Family> trimerFams = Parser.dirToFams(train_dir + "/trimer/", CoilType.TRIMER);
				ArrayList<Sequence> negative_train = Parser.dirToSeqs(train_dir + "/negative_train/");
				ArrayList<Sequence> negative_test = Parser.dirToSeqs(train_dir + "/negative_test/");
				CoilHMM myHMM = new CoilHMM();
				trainAndTest(myHMM, dimerFams,trimerFams,negative_train, negative_test);
			}
		} catch (ParseException exp) {
			System.err.println("Invalid arguments");
		}
	}

	public static void trainAndTest(CoilHMM myHMM, ArrayList<Family> dimerFams, ArrayList<Family>
	trimerFams, ArrayList<Sequence> negative_train, ArrayList<Sequence> negative_test) {
		PredictionResults preds = new PredictionResults();
		for(Family fam: trimerFams) {
			testFam(myHMM, fam, dimerFams,trimerFams,negative_train, preds);
		}
		for(Family fam: dimerFams) {
			testFam(myHMM,fam, dimerFams, trimerFams, negative_train, preds);
		}
		testNegs(myHMM,dimerFams,trimerFams,negative_train,negative_test,preds);
		preds.print(base_dir + "final_results.txt");
	}


	public static void testFam(CoilHMM myHMM, Family fam, ArrayList<Family> dimerFams, ArrayList<Family> trimerFams, 
			ArrayList<Sequence> negatives, PredictionResults res) {
		if(fam.type() == CoilType.DIMER) {
			//Generate a list of the rest of positive samples
			ArrayList<Family> restDimerSamples = ((ArrayList<Family>) dimerFams.clone());
			restDimerSamples.remove(fam);
			try {
				myHMM.generate_hmm2(base_dir, "/leaveOneOutReg_" + fam.toString() + ".dat",
						restDimerSamples, trimerFams, negatives);
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} else if (fam.type() == CoilType.TRIMER) {
			//Generate a list of the rest of positive samples
			ArrayList<Family> restTrimerSamples = ((ArrayList<Family>) trimerFams.clone());
			restTrimerSamples.remove(fam);
			try {
				myHMM.generate_hmm2(base_dir, "/leaveOneOutReg_" + fam.toString() + ".dat",
						dimerFams, restTrimerSamples, negatives);
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		for(Sequence s: fam.getSeqs()) {
			//Make fb predictions
			HashMap<CoilState,double[]> fb_res = Viterbi.forward_backward(myHMM, s.toObservedArray());

			//List of probability of each type at each location
			ArrayList<EnumMap<CoilType,Double>> fb_types = new ArrayList<EnumMap<CoilType,Double>>();
			for(int i=0;i<s.length();i++) {
				EnumMap<CoilType,Double> scores = new EnumMap<CoilType, Double>(CoilType.class);
				for(CoilType t: CoilType.values()) {
					scores.put(t, 0.0);
				}
				for(CoilState state: fb_res.keySet()) {
					double curr = scores.get(state.type);
					scores.put(state.type, curr + fb_res.get(state)[i]);
				}
				fb_types.add(scores);
			}
			res.pos_preds(s,fb_types);
		}
		res.print_last(results_dir + fam);
	}

	public static void print_preds(PrintWriter outputStream, Sequence s, ArrayList<EnumMap<CoilType,Double>> fb_types) {
		//Get most likely oligomerization state at each location:
		String pred = "";
		ArrayList<CoilType> predTypes = new ArrayList<CoilType>();
		for(int i=0;i<fb_types.size();i++) {
			EnumMap<CoilType,Double> type_scores = fb_types.get(i);
			CoilType maxType = null;
			double maxScore = Double.MIN_VALUE;
			for(CoilType type: CoilType.values()){
				if(type_scores.get(type) > maxScore) {
					maxType = type;
					maxScore = type_scores.get(type);
				}
			}
			predTypes.add(maxType);
			pred += maxType.toString();
		}
		outputStream.println(s);
		outputStream.println(pred);
		//outputStream.println(myHMM.scoreCoilEnergy(s.toCoils().get(0)));
	}

	public static void testNegs(CoilHMM myHMM, ArrayList<Family> dimerFams, ArrayList<Family>
	trimerFams, ArrayList<Sequence> negative_train, ArrayList<Sequence> negative_test, PredictionResults preds) {
		try {
			myHMM.generate_hmm2(base_dir,"/leaveOneOutReg_complete.txt",
					dimerFams, trimerFams, negative_train);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		for (Sequence s: negative_test) {
			HashMap<CoilState,double[]> fb_res = Viterbi.forward_backward(myHMM, s.toObservedArray());

			//List of probability of each type at each location
			ArrayList<EnumMap<CoilType,Double>> fb_types = new ArrayList<EnumMap<CoilType,Double>>();
			for(int i=0;i<s.length();i++) {
				EnumMap<CoilType,Double> scores = new EnumMap<CoilType, Double>(CoilType.class);
				for(CoilType t: CoilType.values()) {
					scores.put(t, 0.0);
				}
				for(CoilState state: fb_res.keySet()) {
					double curr = scores.get(state.type);
					scores.put(state.type, curr + fb_res.get(state)[i]);
				}
				fb_types.add(scores);
			}
			preds.neg_preds(s, fb_types);
		}
		preds.print_last(results_dir + "/negs.txt");
	}

	public static void printCoil(CoilHMM myHMM, String path, Sequence s) throws IOException {
		PrintWriter outputStream = new PrintWriter(new FileWriter(path, true));
		outputStream.println("# Sequence name: " + s.name);
		outputStream.println("#");
		outputStream.println("% \tPosition\tResidue\t\tRegister (Dimer,Trimer)\t\tProbability\tDimer Prob\tTrimer Prob\tProbability by Heptad (a,b,c,d,e,f,g)");
		HashMap<CoilState,double[]> fb_res = Viterbi.forward_backward(myHMM, s.toObservedArray());

		for(int i=0;i<s.length();i++) {
			EnumMap<Heptad,Double> hept_scores = new EnumMap<Heptad,Double>(Heptad.class);
	
			EnumMap<Heptad,Double> d_hept_scores = new EnumMap<Heptad,Double>(Heptad.class);
			EnumMap<Heptad,Double> t_hept_scores = new EnumMap<Heptad,Double>(Heptad.class);
			//Stores pseudo-probabilities of each coiltype (normalize later) 
			EnumMap<CoilType, Double> type_scores = new EnumMap<CoilType, Double>(CoilType.class);
			//Initialize enummaps:
			for(CoilType type: CoilType.values()) {
				type_scores.put(type, 0.0);
			}
			for(Heptad h: Heptad.values()) {
				hept_scores.put(h, 0.0);
				d_hept_scores.put(h, 0.0);
				t_hept_scores.put(h, 0.0);
			}
			for(CoilState state: fb_res.keySet()) {
				CoilType type = state.type;
				Double score = fb_res.get(state)[i];
				Heptad h = state.hept;
				type_scores.put(state.type, type_scores.get(state.type) + score);
				//Increment the hept scores with this state
				if(type == CoilType.DIMER) {
					d_hept_scores.put(h, d_hept_scores.get(h)+score);
				}
				if(type == CoilType.TRIMER) {
					t_hept_scores.put(h, t_hept_scores.get(h)+score);
				}
				if(type != CoilType.NONE) {
					hept_scores.put(h,hept_scores.get(h)+score);
				}
			}
			Double total_score = type_scores.get(CoilType.DIMER) + type_scores.get(CoilType.TRIMER) + type_scores.get(CoilType.NONE);
			Double d_score = type_scores.get(CoilType.DIMER) / total_score;
			Double t_score = type_scores.get(CoilType.TRIMER) / total_score;
			Double coil_score = d_score + t_score;

			//Find the most likely heptad for dimer, trimer and overall:
			Heptad max_d_hept = null;
			Double max_d_hept_score = Double.NEGATIVE_INFINITY;
			Heptad max_t_hept = null;
			Double max_t_hept_score = Double.NEGATIVE_INFINITY;
			Heptad max_hept = null;
			Double max_hept_score = Double.NEGATIVE_INFINITY;
			for(Heptad h: Heptad.values()) {
				if(d_hept_scores.get(h) > max_d_hept_score) {
					max_d_hept = h;
					max_d_hept_score = d_hept_scores.get(h);
				}
				if(t_hept_scores.get(h) > max_t_hept_score) {
					max_t_hept = h;
					max_t_hept_score = t_hept_scores.get(h);
				}
				if(hept_scores.get(h) > max_hept_score) {
					max_hept = h;
					max_hept_score = hept_scores.get(h);
				}
			}
			String h_scores = "";
			for(Heptad h: Heptad.values()) {
				int h_sc = (int)(hept_scores.get(h)*1000);
				double h_score = ((double)h_sc)/1000;
				h_scores += h.name() + ":" + h_score + "  ";
			}

			//Round to three decimals:
			int d_sc = (int)(d_score*1000);
			d_score = ((double)d_sc)/1000;

			int t_sc = (int)(t_score*1000);
			t_score = ((double)t_sc)/1000;

			int sc = (int)(coil_score*1000);
			coil_score = ((double)sc)/1000;
			//Formatting: Location | Residue | Hept(dHept,tHept) | 
			//CoilProb | DimCoilProb | TrimCoilProb
			String out = "";
			out += "\t" + Integer.toString(i+1);
			out += "\t\t" + s.getAA(i);
			out += "\t\t" + max_hept.toString();
			out += "(" + max_d_hept.toString() + "," + max_t_hept.toString() + ")";
			out += "\t\t\t\t" + coil_score;
			out += "\t\t" + d_score;
			out += "\t\t" + t_score;
			out += "\t\t" + h_scores;
			outputStream.println(out);
		}
		outputStream.close();
	}
}