package coilPred;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import sequences.CoilType;
import sequences.Family;
import sequences.Sequence;

public class Parser {
	//Grabs a CoiledCoil array out of a directory.
	public static ArrayList<Family> dirToFams(String dir, CoilType type) throws IOException {
		boolean misc = true;
		ArrayList<Family> result = new ArrayList<Family>();
		Family miscFam = null;
		if(misc) {
			miscFam = new Family("misc", type);
			result.add(miscFam);
		}
		//Open directory
		File directory = 
			new File(dir);
		File files[] = directory.listFiles();
		//Iterate through all files in the directory
		for (File f: files) {
			Family thisFam = fileToFam(f,type);
			//If <= 5ccs and misc is true, add to misc instead of creating a new family:
			if (misc && thisFam.famToCoils().size() < 5) {
				miscFam.add(thisFam);
			} else {
				result.add(thisFam);
			}
		}
		return result;
	}

	public static Family fileToFam(File f, CoilType type) throws IOException {
		Family thisFam = new Family(f.getName(), type);
		FileInputStream in = new FileInputStream(f);
		BufferedReader reader = new BufferedReader(new InputStreamReader(in));
		//Check each line to see whether the line is the heptad string
		String residueLine;
		String heptadLine;
		String nextLine = "";
		while (!(nextLine == null)) {
			Pattern p = Pattern.compile("long");
			Matcher m = p.matcher(nextLine);
			if (m.find()) {
				residueLine = reader.readLine();
				heptadLine = reader.readLine();
				Pattern p2 = Pattern.compile("\\w+");
				thisFam.add(new Sequence(type, residueLine, heptadLine));
			}
			nextLine = reader.readLine();
		}
		return thisFam;
	}
	
	public static ArrayList<Sequence> dirToSeqs(String dir) throws IOException {
		ArrayList<Sequence> out = new ArrayList<Sequence>();
		//Open directory
		File directory = 
			new File(dir);
		File files[] = directory.listFiles();
		//Iterate through all files in the directory
		for (File f: files) {
			out.addAll(Parser.fileToSeqs(f));
		}
		return out;
	}
	
	public static ArrayList<Sequence> fileToSeqs(File f) throws IOException {
		FileInputStream in = new FileInputStream(f);
		BufferedReader reader = new BufferedReader(new InputStreamReader(in));
		ArrayList<Sequence> out = new ArrayList<Sequence>();
		//Check each line to see whether it is an amino acid sequence 
		String nextLine = "";
		nextLine = reader.readLine();
		String name = "Unnamed sequence";
		String seq_string = "";
		while(nextLine != null) {
			Pattern p = Pattern.compile("^>");
			Matcher m = p.matcher(nextLine);
			if (m.find()) {
				if (seq_string.length() > 0) {
					out.add(new Sequence(name,seq_string));
				}
				seq_string = "";
				name = nextLine.substring(1);
			}
			p = Pattern.compile("^[A-Z]+\\*?$");
			m = p.matcher(nextLine);
			if (m.find()) {
				//remove (possible) trailing '*'
				if (nextLine.contains("*")) {
					seq_string += nextLine.substring(0,nextLine.length()-1);
				} else {
					seq_string += nextLine;
				}
			}
			nextLine = reader.readLine();
		}
		if (seq_string.length() > 0) {
			out.add(new Sequence(name,seq_string));
		}
		return out;
	}

	/*
	//Grabs the sequences out of a fasta-style file
	public static ArrayList<Sequence> nonCCParseFile(String file) throws IOException {
		// my location: "/afs/athena.mit.edu/user/j/t/jtrigg/Desktop/coiledcoil/coiledcoildata/dimer/parallel/"
		ArrayList<Sequence> coilArray = new ArrayList<Sequence>();
		//Open directory
		//File directory = 
		//	new File(dir);
		//File files[] = directory.listFiles();

		//Iterate through all files in the directory
		//for (File f: files) {
		File f = new File(file);
		FileInputStream in = new FileInputStream(f);
		BufferedReader reader = new BufferedReader(new InputStreamReader(in));
		//Check each line to see whether the line is the heptad string
		String residueLine;
		String nextLine = "";
		while (!(nextLine == null)) {
			Pattern p = Pattern.compile(">");
			Matcher m = p.matcher(nextLine);
			if (m.find()) {
				residueLine = reader.readLine();
				coilArray.add(new Sequence(residueLine));
			}
			nextLine = reader.readLine();
		}
		return coilArray;
	}
	*/

}