package coilPred;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.EnumMap;

import sequences.CoilType;
import sequences.Sequence;

/**
 * Class to store the results from running a prediction algorithm on a number of families
 * @author jtrigg
 *
 */
public class PredictionResults {
	//Initialize the metrics:
	//pRes[i][0] = # positive residues correctly predicted at setting i
	//pRes[i][1] = # positive residues altogether
	//nRes[i][0] = # negative residues correctly predicted at setting i
	//nRes[i][1] = # negative residues altogether
	//etc...
	Double[][] pRes = new Double[21][2];
	Double[][] nRes = new Double[21][2];
	Double[][] pCoil = new Double[21][2];
	Double[][] nCoil = new Double[21][2];
	Double[][] dRes = new Double[21][2];
	Double[][] tRes = new Double[21][2];
	Double[][] dCoil = new Double[21][2];
	Double[][] tCoil = new Double[21][2];
	Double[][] pResLast;
	Double[][] nResLast;
	Double[][] pCoilLast;
	Double[][] nCoilLast;
	Double[][] dResLast;
	Double[][] tResLast;
	Double[][] dCoilLast;
	Double[][] tCoilLast;
	public PredictionResults() {
		for(int i=1;i<=21;i++) {
			dRes[i-1][0] = 0.0;
			dRes[i-1][1] = 0.0;
			tRes[i-1][0] = 0.0;
			tRes[i-1][1] = 0.0;
			dCoil[i-1][0] = 0.0;
			dCoil[i-1][1] = 0.0;
			tCoil[i-1][0] = 0.0;
			tCoil[i-1][1] = 0.0;
			pRes[i-1][0] = 0.0;
			pRes[i-1][1] = 0.0;
			nRes[i-1][0] = 0.0;
			nRes[i-1][1] = 0.0;
			pCoil[i-1][0] = 0.0;
			pCoil[i-1][1] = 0.0;
			nCoil[i-1][0] = 0.0;
			nCoil[i-1][1] = 0.0;
		}
		backup();
	}
	
	public void pos_preds(Sequence s, ArrayList<EnumMap<CoilType,Double>> fb_types) {
		seq_truePos(s,fb_types);
		res_truePos(s,fb_types);
		seq_DT(s,fb_types);
		res_DT(s,fb_types);
	}

	public void neg_preds(Sequence s, ArrayList<EnumMap<CoilType,Double>> fb_types) {
		seq_falsePos(s, fb_types);
		res_falsePos(s, fb_types);
	}

	public void print_last(String file_name) {
		//Print last prints the results since the last call to print last
		//e.g. A(); B(); C(); print_last(); D(); E(); F(); print_last(); print();
		// the first print_last() will print the results from A,B,C
		// the second print_last() will print the results from D,E,F
		// and then print() will print the results from all 6.
		try {
			PrintWriter out = new PrintWriter(file_name);
			out.println("dRes0\tdRes1\ttRes0\ttRes1\tdCoil0\tdCoil1\ttCoil0\ttCoil1\tpRes0\t" +
			"pRes1\tnRes0\tnRes1\tpCoil0\tpCoil1\tnCoil0\tnCoil1");
			for(int i=1;i<=21;i++) {
				String s = "";
				if (i==11) {
					s += "***";
				}
				s += (dRes[i-1][0] - dResLast[i-1][0]) + "\t";
				s += (dRes[i-1][1] - dResLast[i-1][1]) + "\t";
				s += (tRes[i-1][0] - tResLast[i-1][0]) + "\t";
				s += (tRes[i-1][1] - tResLast[i-1][1]) + "\t";
				s += (dCoil[i-1][0] - dCoilLast[i-1][0]) + "\t";
				s += (dCoil[i-1][1] - dCoilLast[i-1][1]) + "\t";
				s += (tCoil[i-1][0] - tCoilLast[i-1][0]) + "\t";
				s += (tCoil[i-1][1] - tCoilLast[i-1][1]) + "\t";
				s += (pRes[i-1][0] - pResLast[i-1][0]) + "\t";
				s += (pRes[i-1][1] - pResLast[i-1][1]) + "\t";
				s += (nRes[i-1][0] - nResLast[i-1][0]) + "\t";
				s += (nRes[i-1][1] - nResLast[i-1][1]) + "\t";
				s += (pCoil[i-1][0] - pCoilLast[i-1][0]) + "\t";
				s += (pCoil[i-1][1] - pCoilLast[i-1][1]) + "\t";
				s += (nCoil[i-1][0] - nCoilLast[i-1][0]) + "\t";
				s += (nCoil[i-1][1] - nCoilLast[i-1][1]);
				out.println(s);
			}
			out.close();
		} catch (IOException e) {
			System.out.println("IOException: " + file_name);
		}
		backup();
	}

	private void backup() {
		pResLast = copy_2d(pRes);
		nResLast = copy_2d(nRes);
		pCoilLast = copy_2d(pCoil);
		nCoilLast = copy_2d(nCoil);
		dResLast = copy_2d(dRes);
		tResLast = copy_2d(tRes);
		dCoilLast = copy_2d(dCoil);
		tCoilLast = copy_2d(tCoil);
	}
	
	public static Double[][] copy_2d(Double[][] preds) {
		Double[][] out = new Double[21][2];
		for(int i=1;i<=21;i++) {
			out[i-1][0] = preds[i-1][0];
			out[i-1][1] = preds[i-1][1];
		}
		return out;
	}
	
	public void print(String file_name) {
		try {
			PrintWriter out = new PrintWriter(file_name);
			out.println("dRes0\tdRes1\ttRes0\ttRes1\tdCoil0\tdCoil1\ttCoil0\ttCoil1\tpRes0\t" +
			"pRes1\tnRes0\tnRes1\tpCoil0\tpCoil1\tnCoil0\tnCoil1");
			for(int i=1;i<=21;i++) {
				String s = "";
				s += dRes[i-1][0] + "\t";
				s += dRes[i-1][1] + "\t";
				s += tRes[i-1][0] + "\t";
				s += tRes[i-1][1] + "\t";
				s += dCoil[i-1][0] + "\t";
				s += dCoil[i-1][1] + "\t";
				s += tCoil[i-1][0] + "\t";
				s += tCoil[i-1][1] + "\t";
				s += pRes[i-1][0] + "\t";
				s += pRes[i-1][1] + "\t";
				s += nRes[i-1][0] + "\t";
				s += nRes[i-1][1] + "\t";
				s += pCoil[i-1][0] + "\t";
				s += pCoil[i-1][1] + "\t";
				s += nCoil[i-1][0] + "\t";
				s += nCoil[i-1][1];
				out.println(s);
			}
			out.close();
		} catch (IOException e) {
			System.out.println("IOException: " + file_name);
		}
	}

	//Test whether we predict a true positive, by the Karl Gutwin criterion:
	//the dimer+trimer probability for each residue in the sequence must exceed a cutoff
	private void seq_truePos(Sequence s, ArrayList<EnumMap<CoilType,Double>> fb_types) {
		boolean out = true;
		for (int i=1; i<=21; i++) {
			double cutoff = Math.pow((1 << i)/2048.0,2.0);
			for (int j=0;j<s.length();j++) {
				if (s.getResidue(j).type() == CoilType.NONE) {continue;}
				double d = fb_types.get(j).get(CoilType.DIMER);
				double t = fb_types.get(j).get(CoilType.TRIMER);
				double n = fb_types.get(j).get(CoilType.NONE);
				if ((d+t)/n < cutoff) {
					out = false;
				}
			}
			if (out) {
				pCoil[i-1][0]++;
			}
			pCoil[i-1][1]++;
		}
	}

	private void seq_falsePos(Sequence s, ArrayList<EnumMap<CoilType,Double>> fb_types) {
		//Code for the new Karl test: for every length 28 window in the sequence, predict positive if
		//the minimum score in the window exceeds the cutoff
		//Equivalent to finding 28 consecutive residues that exceed the threshold
		for(int i=1; i<=21; i++) {
			double cutoff = Math.pow((1 << i)/2048.0,2.0);
			boolean out = false;
			int streak = 0;
			for(int j=0; j<fb_types.size(); j++) {
				double d = fb_types.get(j).get(CoilType.DIMER);
				double t = fb_types.get(j).get(CoilType.TRIMER);
				double n = fb_types.get(j).get(CoilType.NONE);
				if ((d+t)/n < cutoff) {
					streak = 0;
				} else {
					streak += 1;
				}
				if (streak >= 28) {
					out = true;
				}
			}
			if (!out) {
				nCoil[i-1][0]++;
			}
			nCoil[i-1][1]++;
		}
	}

	private void res_truePos(Sequence s, ArrayList<EnumMap<CoilType,Double>> fb_types) {
		for (int i=1; i<=21; i++) {
			//pos vs. neg prediction residue by residue
			for(int j = 0; j<s.length(); j++) {
				if(s.getResidue(j).type() == CoilType.NONE) {continue;}
				else {
					double coilProb = fb_types.get(j).get(CoilType.DIMER) + fb_types.get(j).get(CoilType.TRIMER);
					double nonCoilProb = fb_types.get(j).get(CoilType.NONE);
					//Because of rounding error, sometimes coilProb is greater than 1, which causes problems below:
					if(Math.pow((1 << i)/2048.0,2.0) < (coilProb / (nonCoilProb))) {
						pRes[i-1][0]++;
					}
					pRes[i-1][1]++;
				}
			}
		}
	}

	private void res_falsePos(Sequence s, ArrayList<EnumMap<CoilType,Double>> fb_types) {
		for (int i=1; i<=21; i++) {
			//pos vs. neg prediction residue by residue
			for(int j = 0; j<s.length(); j++) {
				double coilProb = fb_types.get(j).get(CoilType.DIMER) + fb_types.get(j).get(CoilType.TRIMER);
				double nonCoilProb = fb_types.get(j).get(CoilType.NONE);
				//Because of rounding error, sometimes coilProb is greater than 1, which causes problems below:
				if(Math.pow((1 << i)/2048.0,2.0) > (coilProb / (nonCoilProb))) {
					nRes[i-1][0]++;
				}
				nRes[i-1][1]++;
			}
		}
	}


	private void res_DT(Sequence s, ArrayList<EnumMap<CoilType,Double>> fb_types) {
		for(int i=1; i<=21; i++) {
			for (int j=0; j<fb_types.size(); j++) {
				double dProb = fb_types.get(j).get(CoilType.DIMER);
				double tProb = fb_types.get(j).get(CoilType.TRIMER);
				//dimer vs. trimer sequence by sequence
				if(s.getResidue(j).type() == CoilType.NONE) {
					continue;
				}
				if(s.getResidue(j).type() == CoilType.DIMER){
					if(Math.pow((1 << i)/2048.0,.5) < dProb/tProb) {
						dRes[i-1][0]++;
					}
					dRes[i-1][1]++;
				} else if (s.getResidue(j).type() == CoilType.TRIMER) {
					if(Math.pow((1 << i)/2048.0,.5) > dProb/tProb) {
						tRes[i-1][0]++;
					}
					tRes[i-1][1]++;
				}
			}
		}
	}

	private void seq_DT(Sequence s, ArrayList<EnumMap<CoilType,Double>> fb_types) {
		CoilType type = CoilType.NONE;
		double dProb = 0.0;
		double tProb = 0.0;
		double nProb = 0.0;
		double pProb = 0.0;
		for(int i = 0; i<s.length(); i++) {
			if(s.getResidue(i).type() == CoilType.NONE) {continue;}
			else {
				type = s.getResidue(i).type();
				double d = fb_types.get(i).get(CoilType.DIMER);
				double t = fb_types.get(i).get(CoilType.TRIMER);
				double n = fb_types.get(i).get(CoilType.NONE);
				dProb += d / (d+t);
				tProb += t / (d+t);
				nProb += 1 - d - t;
				pProb += d + t;
			}
		}
		for(int i=1; i<=21; i++) {
			if(type == CoilType.DIMER){
				if(Math.pow((1 << i)/2048.0,.5) < dProb/tProb) {
					dCoil[i-1][0]++;
				}
				dCoil[i-1][1]++;
			} else {
				if(Math.pow((1 << i)/2048.0,.5) > dProb/tProb) {
					tCoil[i-1][0]++;
				}
				tCoil[i-1][1]++;
			}
		}
	}
}