package coilPred;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Map;
import java.util.Random;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import sequences.CoilState;
import sequences.CoilType;
import sequences.CoiledCoil;
import sequences.Family;
import sequences.Heptad;
import sequences.Sequence;
import tables.HeptadPairwiseProbTable;
import tables.HeptadResProbTable;
import tables.PairwiseProbTable;
import tables.ResProbTable;


public class Regressor {
	public static void setProbs(CoilHMM myHMM, ArrayList<Family> dimerFams, ArrayList<Family> trimerFams, 
			ArrayList<Sequence> negativeList) {
		HeptadResProbTable dimerFreqs;
		HeptadResProbTable trimerFreqs;

		HeptadPairwiseProbTable dimerPairFreqs;
		HeptadPairwiseProbTable trimerPairFreqs;

		dimerFreqs = Processor.emissionFrequencies(dimerFams, 1);
		trimerFreqs = Processor.emissionFrequencies(trimerFams, 1);
		dimerPairFreqs = Processor.allPairwiseResidueFrequencies(dimerFams, 1);
		trimerPairFreqs = Processor.allPairwiseResidueFrequencies(trimerFams, 1);

		ResProbTable nonCCFreqs = Processor.residueFrequencies(negativeList);
		PairwiseProbTable nonCCPairFreqs = Processor.allPairwiseResidueFrequencies(negativeList);

		if(myHMM==null) myHMM = new CoilHMM();
		myHMM.setProbs(dimerFreqs, trimerFreqs, dimerPairFreqs, trimerPairFreqs, 
				nonCCFreqs, nonCCPairFreqs);
	}


	
	@SuppressWarnings("unchecked")
	public static void createRegressionData(CoilHMM myHMM, String base_dir, String reg_file, ArrayList<Family> dimerList, ArrayList<Family> trimerList, ArrayList<Sequence> negativeList) throws FileNotFoundException {
		File f = new File(base_dir + reg_file); 
		//if we have created this file before, return without doing anything,
		//otherwise generate the file
		if(f.exists()) {return;}
		PrintWriter outputStream = new PrintWriter(base_dir + reg_file);

		int[] gapsUsed = {1,2,3,4,5,6,7};

		//Generate the string to represent the energies used:
		String titleList = "dInd" + "\t"+ "tInd" + "\t"+ "nInd" + "\t" + "Length" + "\t" + 
		"n0";

		titleList += "\t" + "d0";
		titleList += "\t" + "t0";
		titleList += "\t" + "n";
		titleList += "\t" + "d";
		titleList += "\t" + "t";
		for(Heptad h: Heptad.values()) {
			titleList += "\t" + "d0" + h.toString();
			//			titleList += "\t" + "ds0" + h.toString();
			titleList += "\t" + "t0" + h.toString();
		}
		for (int i: gapsUsed) {
			titleList += "\t" + "n" + i;
			titleList += "\t" + "d" + i;
			titleList += "\t" + "t" + i;
		}
		for (Heptad h: Heptad.values()) {
			titleList += "\t" + "hPhob" + h;
		}
		for (Heptad h: Heptad.values()) {
			titleList += "\t" + "charge" + h;
		}
		for (Heptad h: Heptad.values()) {
			titleList += "\t" + "small" + h;
		}
		for (Heptad h: Heptad.values()) {
			titleList += "\t" + "tiny" + h;
		}
		for (Heptad h: Heptad.values()) {
			titleList += "\t" + "branched" + h;
		}
		titleList += "\t" + "sqn0";
		titleList += "\t" + "sqrtn0";

		titleList += "\t" + "seqLength" + "\t" + "famName";
		outputStream.println(titleList);

		for(Family fam: trimerList) {
			ArrayList<Family> trimerFamiliesMinusOne = (ArrayList<Family>) trimerList.clone();
			trimerFamiliesMinusOne.remove(fam);
			setProbs(myHMM,dimerList, trimerFamiliesMinusOne, negativeList);
			for(Sequence seq: fam.getSeqs()) {
				for(CoiledCoil cc: seq.toCoils()) {
					String energyString = myHMM.scoreCoilEnergy(cc);
					double length = cc.length();
					outputStream.println("" + 0.0 + "\t" + 1.0 + "\t" + 0.0 + "\t" + length +"\t" + energyString + "\t" + seq.length() + "\t" + fam.toString());
				}
			}
		}
		for(Family fam: dimerList) {
			ArrayList<Family> dimerFamiliesMinusOne = (ArrayList<Family>) dimerList.clone();
			dimerFamiliesMinusOne.remove(fam);
			setProbs(myHMM, dimerFamiliesMinusOne, trimerList, negativeList);
			for(Sequence seq: fam.getSeqs()) {
				for(CoiledCoil cc: seq.toCoils()) {
					String energyString = myHMM.scoreCoilEnergy(cc);
					double length = cc.length();
					outputStream.println("" + 1.0 + "\t" + 0.0 + "\t" + 0.0 + "\t" + length +"\t" + energyString + "\t" + seq.length() + "\t" + fam.toString());
				}
			}
		}

		
		setProbs(myHMM,dimerList,trimerList,negativeList);

		outputStream.flush();
		String energyString;

		int negCounter=0;
		for(Sequence s: negativeList) {
			negCounter++;
			Random generator = new Random();
			int rand = generator.nextInt(250);
			Heptad randHept = Heptad.a.prev(generator.nextInt(7));
			CoiledCoil cc;
			if (rand < s.length()) {
				cc = s.toCoiledCoil(0, randHept, rand, CoilType.NONE);
				System.out.println(cc);
				energyString = myHMM.scoreCoilEnergy(cc);				
			} else {
				cc = s.toCoiledCoil(0, randHept, s.length(), CoilType.NONE);
				System.out.println(cc);
				energyString = myHMM.scoreCoilEnergy(cc);
			}


			System.out.println(cc);
			outputStream.println("" + 0.0 + "\t" + 0.0 + "\t" + 1.0 + "\t" + cc.length() +"\t" + energyString + "\t" + s.length());
			outputStream.flush();
		}
		outputStream.println("Done");
		outputStream.close();
	}
	

	@SuppressWarnings("unchecked")
	public static void createRegressionData2(CoilHMM myHMM, String base_dir, String reg_file, ArrayList<Family> dimerList, ArrayList<Family> trimerList, ArrayList<Sequence> negativeList) throws FileNotFoundException {
		File f = new File(base_dir + reg_file); 
		//if we have created this file before, return without doing anything,
		//otherwise generate the file
		if(f.exists()) {return;}
		PrintWriter outputStream = new PrintWriter(base_dir + reg_file);

		int[] gapsUsed = {1,2,3,4,5,6,7};

		//Generate the string to represent the energies used:
		String titleList = "dInd" + "\t"+ "tInd" + "\t"+ "nInd";
		
		titleList += "\t" +	"n0";
		titleList += "\t" + "d0";
		titleList += "\t" + "t0";
		titleList += "\t" + "n";
		titleList += "\t" + "d";
		titleList += "\t" + "t";
		titleList += "\t" + "hPhoba";
		titleList += "\t" + "hPhobd";
		outputStream.println(titleList);

		for(Family fam: trimerList) {
			ArrayList<Family> trimerFamiliesMinusOne = (ArrayList<Family>) trimerList.clone();
			trimerFamiliesMinusOne.remove(fam);
			setProbs(myHMM,dimerList, trimerFamiliesMinusOne, negativeList);
			for(Sequence seq: fam.getSeqs()) {
				for(CoiledCoil cc: seq.toCoils()) {
					String energyString = myHMM.scoreCoilEnergy2(cc);
					outputStream.println("" + 0.0 + "\t" + 1.0 + "\t" + 0.0 + "\t" + energyString);
				}
			}
		}
		for(Family fam: dimerList) {
			ArrayList<Family> dimerFamiliesMinusOne = (ArrayList<Family>) dimerList.clone();
			dimerFamiliesMinusOne.remove(fam);
			setProbs(myHMM, dimerFamiliesMinusOne, trimerList, negativeList);
			for(Sequence seq: fam.getSeqs()) {
				for(CoiledCoil cc: seq.toCoils()) {
					String energyString = myHMM.scoreCoilEnergy2(cc);
					outputStream.println("" + 1.0 + "\t" + 0.0 + "\t" + 0.0 + "\t" + energyString);
				}
			}
		}

		setProbs(myHMM,dimerList,trimerList,negativeList);

		outputStream.flush();
		String energyString;

		int negCounter=0;
		for(Sequence s: negativeList) {
			negCounter++;
			Random generator = new Random();
			int rand = generator.nextInt(250);
			Heptad randHept = Heptad.a.prev(generator.nextInt(7));
			CoiledCoil cc;
			if (rand < s.length()) {
				cc = s.toCoiledCoil(0, randHept, rand, CoilType.NONE);
				System.out.println(cc);
				energyString = myHMM.scoreCoilEnergy2(cc);				
			} else {
				cc = s.toCoiledCoil(0, randHept, s.length(), CoilType.NONE);
				System.out.println(cc);
				energyString = myHMM.scoreCoilEnergy2(cc);
			}


			System.out.println(cc);
			outputStream.println("" + 0.0 + "\t" + 0.0 + "\t" + 1.0 + "\t" + energyString);
			outputStream.flush();
		}
		outputStream.println("Done");
		outputStream.close();
	}

	
	//Look in fileName for regressionData file, otherwise create it ourselves...
	public static ArrayList<Double> getRegressionCoeffs(CoilHMM myHMM, String base_dir, String reg_file, ArrayList<Family> dimerFams, ArrayList<Family> trimerFams,
			ArrayList<Sequence> negativeList)
			throws FileNotFoundException, InterruptedException, IOException{
		PrintWriter outputStream = new PrintWriter(base_dir+"/parseRegression.do");
		String parseRegBase = 
			"insheet using " + base_dir + reg_file +
			"\ndrop if dind==\"Done\" " +
			"\ndestring,replace " +
			"\ngen ccInd = dind + 2*tind " +
			"\nby ccInd, sort: gen pop = _N	" +
			"\nby famname, sort: gen famSize = _N" +
			"\nbysort famname ccInd: gen famNums = _n==1" +
			"\nby ccInd,sort: replace famNums = sum(famNums)" +
			"\nby ccInd: replace famNums = famNums[_N]" +
			"\ngen popinv = 1/pop " +
			"\ngen noPopInv = 1-.9*nind" +
			"\ngen newpopinv = popinv*(1+3*tind)" +
			"\ngen modpopinv = popinv*(1+999*nind) " +
			"\ngen newWeight = 1/(famSize*famNums)" +
			"\ngen tp = t1 + t3 + t6"+
			"\ngen dp = d6 + d7"+
			"\ngen d0x = d0a+d0d"+
			"\ngen t0x = t0a+t0d"+
			"\ngen d0y = d0b + d0c"+
			"\ngen t0y = t0b+t0c"+
			"\ngen d0z = d0e+d0f+d0g"+
			"\ngen t0z = t0e+t0f+t0g" +
			"\ngen d0w = d0e+d0g"+
			"\ngen t0w = t0e+t0g" +
			"\ngen longer = length>=28" +
			"\nconstraint define 1 [1]n = [2]n " +
			"\nconstraint define 2 [2]t0 = 0" +
			"\nconstraint define 5 [1]hphobd = 0" +
			"\nconstraint define 6 [2]d = 0" +
			"\nconstraint define 7 [1]hphoba = 0" +
			"\nconstraint define 9 [1]d = 0" +
			"\nconstraint define 10 [2]t = 0" +
			"\nconstraint define 12 [2]d = 0" + 
			"\nconstraint define 13 [1]n = 0" +
			"\nconstraint define 14 [2]n = 0" +
			"\nconstraint define 15 [1]dp = 0" +
			"\nconstraint define 16 [2]dp = 0" +
			"\nconstraint define 17 [1]tp = 0" +
			"\nconstraint define 18 [2]tp = 0" +
			"\nconstraint define 19 [1]hphobd = 0" +
			"\nconstraint define 20 [2]hphobd = 0" +
			"\nconstraint define 21 [1]d0z = 0" +
			"\nconstraint define 22 [2]d0z = 0" +
			"\nconstraint define 23 [1]t0y = [1]t0z" +
			"\nconstraint define 24 [2]t0y = [2]t0z" +
			"\nconstraint define 25 [1]smallg = 0" +
			"\nconstraint define 26 [2]smallg = 0" +
			"\nconstraint define 27 [1]d1 = 0" +
			"\nconstraint define 28 [2]d1 = 0" +
			"\nconstraint define 29 [1]d2 = 0" +
			"\nconstraint define 30 [2]d2 = 0" +
			"\nconstraint define 31 [1]d3 = 0" +
			"\nconstraint define 32 [2]d3 = 0" +
			"\nconstraint define 33 [1]d4 = 0" +
			"\nconstraint define 34 [2]d4 = 0" +
			"\nconstraint define 35 [1]d5 = 0" +
			"\nconstraint define 36 [2]d5 = 0" +
			"\nconstraint define 37 [1]d6 = 0" +
			"\nconstraint define 38 [2]d6 = 0" +
			"\nconstraint define 39 [1]d7 = 0" +
			"\nconstraint define 40 [2]d7 = 0" +
			"\nconstraint define 41 [1]t1 = 0" +
			"\nconstraint define 42 [2]t1 = 0" +
			"\nconstraint define 43 [1]t2 = 0" +
			"\nconstraint define 44 [2]t2 = 0" +
			"\nconstraint define 45 [1]t3 = 0" +
			"\nconstraint define 46 [2]t3 = 0" +
			"\nconstraint define 47 [1]t4 = 0" +
			"\nconstraint define 48 [2]t4 = 0" +
			"\nconstraint define 49 [1]t5 = 0" +
			"\nconstraint define 50 [2]t5 = 0" +
			"\nconstraint define 51 [1]t6 = 0" +
			"\nconstraint define 52 [2]t6 = 0" +
			"\nconstraint define 53 [1]t7 = 0" +
			"\nconstraint define 54 [2]t7 = 0" +
			"\nconstraint define 55 [1]hphoba = 0" +
			"\nconstraint define 56 [2]hphoba = 0" +
			"\nconstraint define 57 [1]hphobd = 0" +
			"\nconstraint define 58 [2]hphobd = 0" +
			"\nconstraint define 59 [1]d0a = [1]d0d" +
			"\nconstraint define 60 [2]d0a = [2]d0d" +

			"\nconstraint define 61 [1]d0e = [1]d0g" +
			"\nconstraint define 62 [2]d0e = [2]d0g" +

			"\nconstraint define 63 [1]t0a = [1]t0d" +
			"\nconstraint define 64 [2]t0a = [2]t0d" +

			"\nconstraint define 65 [1]t0e = [1]t0g" +
			"\nconstraint define 66 [2]t0e = [2]t0g" +

			"\nconstraint define 67 [1]d0d = [1]d0e" +
			"\nconstraint define 68 [2]d0d = [2]d0e" +

			"\nconstraint define 69 [1]t0d = [1]t0e" +
			"\nconstraint define 70 [2]t0d = [2]t0e" +

			"\nconstraint define 71 [1]d0b = 0" +
			"\nconstraint define 72 [2]d0b = 0" +
			"\nconstraint define 73 [1]d0c = 0" +
			"\nconstraint define 74 [2]d0c = 0" +
			"\nconstraint define 75 [1]d0f = 0" +
			"\nconstraint define 76 [2]d0f = 0" +

			"\nconstraint define 77 [1]t0b = 0" +
			"\nconstraint define 78 [2]t0b = 0" +
			"\nconstraint define 79 [1]t0c = 0" +
			"\nconstraint define 80 [2]t0c = 0" +
			"\nconstraint define 81 [1]t0f = 0" +
			"\nconstraint define 82 [2]t0f = 0" +


			"\nconstraint define 83 [2]d0e = 0" +
			"\nconstraint define 84 [1]d0e = 0" +
			"\nconstraint define 85 [2]d0g = 0" +
			"\nconstraint define 86 [1]d0g = 0" +



			"\nconstraint define 87 [2]t0e = 0" +
			"\nconstraint define 88 [1]t0e = 0" +
			"\nconstraint define 89 [2]t0g = 0" +
			"\nconstraint define 90 [1]t0g = 0" +

			"\nconstraint define 91 [1]d0a = 0" +
			"\nconstraint define 92 [1]d0a = 0" +

			"\nconstraint define 93 [1]d0d = 0" +
			"\nconstraint define 94 [1]d0d = 0" +

			"\nconstraint define 95 [1]t0a = 0" +
			"\nconstraint define 96 [2]t0a = 0" +

			"\nconstraint define 97 [1]t0d = 0" +
			"\nconstraint define 98 [1]t0d = 0" +
			"\nconstraint define 100 [1]d1 = [1]d2" +
			"\nconstraint define 101 [1]d2 = [1]d3" +
			"\nconstraint define 102 [1]d3 = [1]d4" +
			"\nconstraint define 103 [1]d4 = [1]d5" +
			"\nconstraint define 104 [1]d5 = [1]d6" +
			"\nconstraint define 105 [1]d6 = [1]d7" +
			"\nconstraint define 106 [2]d1 = [2]d2" +
			"\nconstraint define 107 [2]d2 = [2]d3" +
			"\nconstraint define 108 [2]d3 = [2]d4" +
			"\nconstraint define 109 [2]d4 = [2]d5" +
			"\nconstraint define 110 [2]d5 = [2]d6" +
			"\nconstraint define 111 [2]d6 = [2]d7" +
			"\nconstraint define 112 [1]t1 = [1]t2" +
			"\nconstraint define 113 [1]t2 = [1]t3" +
			"\nconstraint define 114 [1]t3 = [1]t4" +
			"\nconstraint define 115 [1]t4 = [1]t5" +
			"\nconstraint define 116 [1]t5 = [1]t6" +
			"\nconstraint define 117 [1]t6 = [1]t7" +
			"\nconstraint define 118 [2]t1 = [2]t2" +
			"\nconstraint define 119 [2]t2 = [2]t3" +
			"\nconstraint define 120 [2]t3 = [2]t4" +
			"\nconstraint define 121 [2]t4 = [2]t5" +
			"\nconstraint define 122 [2]t5 = [2]t6" +
			"\nconstraint define 123 [2]t6 = [2]t7" +
			"\nconstraint define 124 [1]d0a = [1]d0b" +
			"\nconstraint define 125 [1]d0b = [1]d0c" +
			"\nconstraint define 126 [1]d0c = [1]d0d" +
			"\nconstraint define 127 [1]d0d = [1]d0e" +
			"\nconstraint define 128 [1]d0e = [1]d0f" +
			"\nconstraint define 129 [1]d0f = [1]d0g" +
			"\nconstraint define 130 [2]d0a = [2]d0b" +
			"\nconstraint define 131 [2]d0b = [2]d0c" +
			"\nconstraint define 132 [2]d0c = [2]d0d" +
			"\nconstraint define 133 [2]d0d = [2]d0e" +
			"\nconstraint define 134 [2]d0e = [2]d0f" +
			"\nconstraint define 135 [2]d0f = [2]d0g" +
			"\nconstraint define 136 [1]t0a = [1]t0b" +
			"\nconstraint define 137 [1]t0b = [1]t0c" +
			"\nconstraint define 138 [1]t0c = [1]t0d" +
			"\nconstraint define 139 [1]t0d = [1]t0e" +
			"\nconstraint define 140 [1]t0e = [1]t0f" +
			"\nconstraint define 141 [1]t0f = [1]t0g" +
			"\nconstraint define 142 [2]t0a = [2]t0b" +
			"\nconstraint define 143 [2]t0b = [2]t0c" +
			"\nconstraint define 144 [2]t0c = [2]t0d" +
			"\nconstraint define 145 [2]t0d = [2]t0e" +
			"\nconstraint define 146 [2]t0e = [2]t0f" +
			"\nconstraint define 147 [2]t0f = [2]t0g" +
			"\nconstraint define 148 [1]d0c = [1]d0e" +
			"\nconstraint define 149 [2]d0c = [2]d0e" +
			"\nconstraint define 150 [1]t0c = [1]t0e" +
			"\nconstraint define 151 [2]t0c = [2]t0e" +
			"\nconstraint define 152 [1]hphobe = 0" +
			"\nconstraint define 153 [2]hphobe = 0" +
			"\nconstraint define 154 [1]hphobg = 0" +
			"\nconstraint define 155 [2]hphobg = 0" +
			"\nconstraint define 156 [1]n0 = 0" +
			"\nconstraint define 157 [2]n0 = 0" +
			"\nconstraint define 158 [1]d0a = 0" +
			"\nconstraint define 159 [2]d0a = 0" +
			"\nconstraint define 160 [1]d0b = 0" +
			"\nconstraint define 161 [2]d0b = 0" +
			"\nconstraint define 162 [1]d0c = 0" +
			"\nconstraint define 163 [2]d0c = 0" +
			"\nconstraint define 164 [1]d0d = 0" +
			"\nconstraint define 165 [2]d0d = 0" +
			"\nconstraint define 166 [1]d0e = 0" +
			"\nconstraint define 167 [2]d0e = 0" +
			"\nconstraint define 168 [1]d0f = 0" +
			"\nconstraint define 169 [2]d0f = 0" +
			"\nconstraint define 170 [1]d0g = 0" +
			"\nconstraint define 171 [2]t0g = 0" +
			"\nconstraint define 172 [1]t0a = 0" +
			"\nconstraint define 173 [2]t0a = 0" +
			"\nconstraint define 174 [1]t0b = 0" +
			"\nconstraint define 175 [2]t0b = 0" +
			"\nconstraint define 176 [1]t0c = 0" +
			"\nconstraint define 177 [2]t0c = 0" +
			"\nconstraint define 178 [1]t0d = 0" +
			"\nconstraint define 179 [2]t0d = 0" +
			"\nconstraint define 180 [1]t0e = 0" +
			"\nconstraint define 181 [2]t0e = 0" +
			"\nconstraint define 182 [1]t0f = 0" +
			"\nconstraint define 183 [2]t0f = 0" +
			"\nconstraint define 184 [1]t0g = 0" +
			"\nconstraint define 185 [2]t0g = 0";
		String constraint_str = " constraints(";

		
		boolean d0_t0 = true;
		boolean d0_t0_d0a_d0d_t0a_t0d = false;

		boolean a_d_equal = false;
		boolean e_g_equal = false;
		boolean d_e_equal = false;
		boolean no_b_c_f = false;

		//no dimer, trimer pairs
		boolean no_pairs = false;
		boolean eq_pairs = true;

		//hphob
		boolean no_hphob_a_d = false;
		boolean no_hphob_e_g = true;

		//Overrides the above with a default setting
		//def is the specification n0, n, d, t, d0, t0, hphoba, hphobd
		boolean def = true;
		boolean thesis_version = false;
		//Deal with n, n0


		//Deal with d0, t0
		if (d0_t0) {
			for(int i = 124; i < 148; i++) {
				constraint_str += " " + String.valueOf(i);
			}
		} else if (d0_t0_d0a_d0d_t0a_t0d){
			constraint_str += " 125 128 129 131 134 135 137 140 141" +
			" 143 146 147 148 149 150 151";
			if (a_d_equal) {
				constraint_str += " 59 60 63 64";				
			}
		} else {
			if (a_d_equal) {
				constraint_str += " 59 60 63 64";
			}
			if (e_g_equal) {
				constraint_str += " 61 62 65 66";
			}
			if (d_e_equal) {
				constraint_str += " 67 68 69 70";
			}
			if (no_b_c_f) {
				for(int i = 71; i < 83; i++) {
					constraint_str += " " + String.valueOf(i);
				}
			}
		}

		//Deal with d, t
		if (no_pairs) {
			for(int i = 27; i < 55; i++) {
				constraint_str += " " + String.valueOf(i);
			}
		} else if (eq_pairs) {
			for(int i = 100; i < 124; i++) {
				constraint_str += " " + String.valueOf(i);
			}
		}

		//Deal with hphoba, hphobd
		if (no_hphob_a_d) {
			constraint_str += " 55 56 57 58";
		}
		if (no_hphob_e_g) {
			constraint_str += " 152 153 154 155";
		}




		constraint_str += ")";

		if (def) {
			constraint_str = " constraints(";

			for (int i = 100; i < 148; i++) {
				constraint_str += " " + String.valueOf(i);
			}
//			
//			for (int i = 136; i < 148; i++) {
//				constraint_str += " " + String.valueOf(i);
//			}
//
//			for(int i = 172; i < 186; i++) {
//				constraint_str += " " + String.valueOf(i);
//			}
  
			constraint_str += " 152 153 154 155";
			constraint_str += ")";
		}

		if (thesis_version) {
			constraint_str = " constraints(15,16,21,22,23,24,25,26)";
		}

		outputStream.write(
				parseRegBase +
				"\nmlogit ccInd n0 n d1 d2 d3 d4 d5 d6 d7 t1 t2 t3 t4 t5 t6 t7 " +
				"hphoba hphobd hphobe hphobg " +
				"d0a d0b d0c d0d d0e d0f d0g t0a t0b t0c t0d t0e t0f t0g " +
				"[pweight=popinv]" +
				"," +
				constraint_str +
				" base(0)" +
		"\n");
		outputStream.flush();
		outputStream.close();
		createRegressionData(myHMM, base_dir, reg_file, dimerFams, trimerFams, negativeList);
		ProcessBuilder shellTest = new ProcessBuilder(base_dir + "/bash.sh");
		//Might need to remove the final '\' from base_dir for the next line
		shellTest.directory(new File(base_dir));
		Map<String, String> environment = shellTest.environment();
		environment.put("path", ";"); // Clearing the path variable;
		environment.put("path", base_dir + ";");
		shellTest.start().waitFor();
		File myFile = new File(base_dir + "parseRegression.log");
		ArrayList<Double> result = parseStata(myFile);
		myFile.delete();
		return result;
	}
	
	
	
	//Look in fileName for regressionData file, otherwise create it ourselves...
	public static ArrayList<Double> getRegressionCoeffs2(CoilHMM myHMM, String base_dir, String reg_file, ArrayList<Family> dimerFams, ArrayList<Family> trimerFams,
			ArrayList<Sequence> negativeList)
			throws FileNotFoundException, InterruptedException, IOException{
		PrintWriter outputStream = new PrintWriter(base_dir+"/parseRegression.do");
		String parseRegBase = 
			"insheet using " + base_dir + reg_file +
			"\ndrop if dind==\"Done\" " +
			"\ndestring,replace " +
			"\ngen ccInd = dind + 2*tind " +
			"\nby ccInd, sort: gen pop = _N	" +
			"\ngen popinv = 1/pop " +
			"\ngen noPopInv = 1-.9*nind" +
			"\ngen newpopinv = popinv*(1+3*tind)" +
			"\ngen modpopinv = popinv*(1+999*nind) " +
			"\nconstraint define 1 [1]d0 = 0" +
			"\nconstraint define 2 [2]d0 = 0" +
			"\nconstraint define 3 [1]t0 = 0" +
			"\nconstraint define 4 [2]t0 = 0" +
			"\nconstraint define 5 [1]n0 = 0" +
			"\nconstraint define 6 [2]n0 = 0" +
			"\nconstraint define 7 [1]d = 0" +
			"\nconstraint define 8 [2]d = 0" +
			"\nconstraint define 9 [1]t = 0" +
			"\nconstraint define 10 [2]t = 0" +
			"\nconstraint define 11 [1]n = 0" +
			"\nconstraint define 12 [2]n = 0" +
			"\nconstraint define 13 [1]hphoba = 0" +
			"\nconstraint define 14 [2]hphoba = 0" +
			"\nconstraint define 15 [1]hphobd = 0" +
			"\nconstraint define 16 [2]hphobd = 0";
		String constraint_str = " constraints(";

		
		//Variables for inclusion of each predictor:
		boolean d0 = true;
		boolean t0 = true;
		boolean n0 = true;
		boolean d = true;
		boolean t = true;
		boolean n = true;
		boolean hphoba = true;
		boolean hphobd = true;

		if (!d0) {
			constraint_str += " 1 2";
		}
		if (!t0) {
			constraint_str += " 3 4";
		}
		if (!n0) {
			constraint_str += " 5 6";
		}
		if (!d) {
			constraint_str += " 7 8";
		}
		if (!t) {
			constraint_str += " 9 10";
		}
		if (!n) {
			constraint_str += " 11 12";
		}
		if (!hphoba) {
			constraint_str += " 13 14";
		}
		if (!hphobd) {
			constraint_str += " 15 16";
		}
		constraint_str += ")";

		outputStream.write(
				parseRegBase +
				"\nmlogit ccInd d0 t0 n0 d t n hphoba hphobd " +
				"[pweight=popinv]" +
				"," +
				constraint_str +
				" iterate(100) base(0)" +
		"\n");
		outputStream.flush();
		outputStream.close();
		createRegressionData2(myHMM, base_dir, reg_file, dimerFams, trimerFams, negativeList);
		ProcessBuilder shellTest = new ProcessBuilder(base_dir + "/bash.sh");
		//Might need to remove the final '\' from base_dir for the next line
		shellTest.directory(new File(base_dir));
		Map<String, String> environment = shellTest.environment();
		environment.put("path", ";"); // Clearing the path variable;
		environment.put("path", base_dir + ";");
		shellTest.start().waitFor();
		File myFile = new File(base_dir + "parseRegression.log");
		ArrayList<Double> result = parseStata(myFile);
		myFile.delete();
		return result;
	}

	
	//Gets the coefficients out of a STATA multinomial logit regression:
	public static ArrayList<Double> parseStata(File f) throws IOException {
		FileInputStream in = new FileInputStream(f);
		BufferedReader reader = new BufferedReader(new InputStreamReader(in));
		ArrayList<Double> result = new ArrayList<Double>();
		String nextLine = "";
		//Pattern finds |   (-).#####
		Pattern p1 = Pattern.compile("\\|\\s+\\-?\\d*\\.\\d+");
		//Pattern finds the (-).##### part of the above
		Pattern p2 = Pattern.compile("\\-?\\d*\\.\\d+");
		//A pattern to search for omitted variables, which we will record as having
		//coefficients = 0
		Pattern p3 = Pattern.compile("\\|\\s+\\(omitted\\)");
		Pattern p4 = Pattern.compile("\\|.+(e-)");
		do {
			Matcher m1 = p1.matcher(nextLine);
			Matcher m3 = p3.matcher(nextLine);
			Matcher m4 = p4.matcher(nextLine);
			boolean sci_not = m4.find();
			if(m1.find() && !sci_not) {
				Matcher m2 = p2.matcher(m1.group());
				m2.find();
				result.add(new Double(m2.group()));
			} else if(m3.find() || sci_not) {
				result.add(0.0);
			}
			nextLine = reader.readLine();
		} while(nextLine != null);
		return result;
	}

}