package sequences;

import java.util.ArrayList;

/**
 * 
 * @author jtrigg
 * Stores the data of a coiled-coil
 *
 */
public class CoiledCoil{
	private String heptadString = "";
	private String residueString = "";
	private ArrayList<CoilResidue> residueList = new ArrayList<CoilResidue>();
	
	public CoiledCoil() {
	}
	
	public CoiledCoil(CoilType type, String residues, String heptads) throws InvalidAminoAcidException {
		residueList = new ArrayList<CoilResidue>();
		for (int i = 0; i<residues.length(); i++) {
			if(residues.charAt(i) == 'X') throw (new InvalidAminoAcidException());
			AminoAcid aa = AminoAcid.valueOf("" + residues.charAt(i));
			Heptad h = Heptad.valueOf("" + heptads.charAt(i));
			residueList.add(new CoilResidue(type, aa, h));
		}
		this.residueString = residues;
		this.heptadString = heptads;
	}
	
	public void add(CoilType type, AminoAcid aa, Heptad h) {
		residueList.add(new CoilResidue(type, aa, h));
		heptadString += h.toString();
		residueString += aa.toString();
	}
	
	public ArrayList<AminoAcid> toObservedArray() {
		ArrayList<AminoAcid> result = new ArrayList<AminoAcid>();
		for(CoilResidue res: residueList) {
			result.add(res.getAA());
		}
		return result;
	}
	
	@Override
	public String toString() {
		return residueString + " \n " + heptadString;
	}

	public String heptadString() {
		return heptadString;
	}

	public String residueString() {
		return residueString;
	}

	public int length() {
		return heptadString.length();
	}

	public ArrayList<CoilResidue> residueList() {
		return residueList;
	}
	
	public AminoAcid residue(int i) {
		return residueList.get(i).aminoAcid;
	}
	
	public Heptad heptad(int i) {
		return residueList.get(i).heptadPosition;
	}
}