package tables;

import java.util.HashMap;
import java.util.Set;

public class HMMPosProbTable<E> {
	HashMap<Integer, HashMap<E, Double>> table;
	
	public HMMPosProbTable(HashMap<Integer, HashMap<E, Double>> table) {
		this.table = normalize(table);
	}
	
	public double prob(int index, Set<E> stateSet) {
		double sum = 0;
		for(E e: stateSet) {
			sum += table.get(index).get(e);
		}
		return sum;
	}
	
	public int length() {
		return table.size();
	}
	
	public E maxState(int index) {
		HashMap<E, Double> subTable = table.get(index);
		E maxState = null;
		double maxProb = 0;
		for (E e: subTable.keySet()) {
			if (subTable.get(e) > maxProb) {
				maxState = e;
				maxProb = subTable.get(e);
			}
		}
		return maxState;
	}
	
	public HashMap<E, Double> get(int index) {
		return table.get(index);
	}
	
	public double get(int index, E state) {
		return table.get(index).get(state);
	}
	
	public double maxProb(int index) {
		return table.get(index).get(maxState(index));
	}
	
	private HashMap<Integer, HashMap<E, Double>> normalize(HashMap<Integer, HashMap<E, Double>> table) {
		HashMap<Integer, HashMap<E, Double>> result = new HashMap<Integer, HashMap<E, Double>>();
		for(int i: table.keySet()) {
			double sum = 0;
			HashMap<E, Double> subTable = new HashMap<E, Double>();
			for(E e: table.get(i).keySet()) {
				sum += table.get(i).get(e);
			}
			for(E e: table.get(i).keySet()) {
				subTable.put(e, table.get(i).get(e)/sum);
			}
			result.put(i, subTable);
		}
		return result;
	}
}
